@@ -13,17 +13,16 @@
 import abc
 import atexit
 import logging
+import queue
+import sys
 import threading
-
-import six
-from six.moves import queue as Queue
+import traceback
 
 LOG = logging.getLogger(__name__)
 STOP_EVENT = ("STOP", None, None, None)
 
 
-@six.add_metaclass(abc.ABCMeta)
-class RowEvent(object):
+class RowEvent(object, metaclass=abc.ABCMeta):
     ROW_CREATE = "create"
     ROW_UPDATE = "update"
     ROW_DELETE = "delete"
@@ -98,7 +97,7 @@ class RowEventHandler(object):
     def __init__(self):
         self.__watched_events = set()
         self.__lock = threading.Lock()
-        self.notifications = Queue.Queue()
+        self.notifications = queue.Queue()
         self.notify_thread = threading.Thread(target=self.notify_loop)
         self.notify_thread.daemon = True
         atexit.register(self.shutdown)
@@ -107,10 +106,27 @@ class RowEventHandler(object):
     def start(self):
         self.notify_thread.start()
 
+    @staticmethod
+    def match(candidate, event, row, updates):
+        try:
+            return candidate.matches(event, row, updates)
+        except Exception:
+            # It seems reasonable to treat an Exception as a match failure
+            # as for ROW_UPDATE events old is None and users often check
+            # old.external_ids etc. to see if something has changed. We
+            # definitely don't want to interupt RowEventHandler.matching_events
+            # and miss checking other events due to an exception in this event.
+            # To avoid spamming full Exceptions for expected behavior, just
+            # log the initial location of the Exception.
+            _, _, tb = sys.exc_info()
+            LOG.debug("Event not matched due to exception:\n%s",
+                      traceback.format_tb(tb)[-1])
+            return False
+
     def matching_events(self, event, row, updates):
         with self.__lock:
             return tuple(t for t in self.__watched_events
-                         if t.matches(event, row, updates))
+                         if self.match(t, event, row, updates))
 
     def watch_event(self, event):
         with self.__lock:
@@ -150,17 +166,6 @@ class RowEventHandler(object):
                 LOG.exception('Unexpected exception in notify_loop')
 
     def notify(self, event, row, updates=None):
-        """Method for calling backend to call for each DB update
-
-        :param event:   Backend representation of event type, e.g.
-                        create, update, delete
-        :param row:     Backend representation of a Row object. If it is not
-                        immutable, it should be converted or guaranteed not to
-                        be changed in other threads.
-        :param updates: Backend representation of updates to a Row. e.g.
-                        a Row object with just changed attributes, a
-                        dictionary of changes, etc.
-        """
         matching = self.matching_events(
             event, row, updates)
         for match in matching:
